/* $Id$ */

/* テキスト描画全般を実現する為のベースモジュール */

/* 仕様:
   - ttfフォントを一種類のみ利用できる。
   -- その他の画像等を含める事は行わない(少なくとも、今のところは)
   -- 途中でフォントを変更したりは、今のところは仕様外とする。
      (可能かも知れないが、意図して実装はしない、まだ)
   - 指定されたサーフィスに対し、文字を表示させるだけとする。
   -- 背景やウィンドウの枠が欲しい場合や、カーソル点滅等を実現したい場合は、
      別途、このウィンドウの前後に別のサーフィスを表示させる事。
   - いくつかの組み込みの制御コードを持つ。
   -- 仕様はまだ未定。
   -- とりあえず「\r」はnop、「\n」は改行とする。
   - 折り返しは無し。必要な場合はこの上のレイヤーで実装する事。
   - スクロール制御も無し。サーフィスをあふれる分はそのままあふれる。
*/


module ddddd.textarea;

public import ddddd.importer.public_;
private import ddddd.importer.private_;
private import ddddd.image;
private import std.string;

public:

struct TextArea {
  SDL_Surface *surface; // 本体のサーフィス
  SDL_Color fgcolor; // 前景色(unusedはa扱い/不要なら255を入れておく事)
  SDL_Color bgcolor; // 背景色(unusedはa扱い/不要なら255を入れておく事)
  TTF_Font *font; // 描画フォント
  int locate_x; // 次の文字を描画する為のx座標(ドット単位)
  int locate_y; // 次の文字を描画する為のy座標(ドット単位)
  int linespace; // 行間のドット数
  Uint32 flags; // 各種のフラグ
};

enum {
  RENDERTYPE_UNUSED  = 0,
  RENDERTYPE_SOLID   = 1,
  RENDERTYPE_SHADED  = 2,
  RENDERTYPE_BLENDED = 3,
};

TextArea *ddddd_ta_create(int width, int height,
    SDL_Color fgcolor, SDL_Color bgcolor, TTF_Font *font,
    int linespace, Uint32 surface_flags, Uint32 ta_flags = RENDERTYPE_SHADED)
{
  TextArea *ta = new TextArea;
  RGBAMask *m = ddddd_get_rgbamask();
  ta.surface = SDL_CreateRGBSurface(surface_flags, width, height, ddddd_rw_depth, m.r, m.g, m.b, m.a);
  ta.fgcolor = fgcolor;
  ta.bgcolor = bgcolor;
  ta.font = font;
  ta.linespace = linespace;
  ta.flags = ta_flags;
  ddddd_ta_clear(ta);
  return ta;
}


Uint32 ddddd_ta_color2uint32(TextArea *ta, SDL_Color color)
{
  if (color.unused == 255) {
    return SDL_MapRGB(ta.surface.format, color.r, color.g, color.b);
  }
  else {
    return SDL_MapRGBA(ta.surface.format, color.r, color.g, color.b, color.unused);
  }
}


void ddddd_ta_clear(TextArea *ta)
{
  ta.locate_x = 0;
  ta.locate_y = 0;
  SDL_FillRect(ta.surface, null, ddddd_ta_color2uint32(ta, ta.bgcolor));
  return;
}

void ddddd_ta_lf(TextArea *ta)
{
  //int height = TTF_FontHeight(ta.font); // どっちが良いのか謎
  int height = TTF_FontLineSkip(ta.font);
  ta.locate_x = 0;
  ta.locate_y += height + ta.linespace;
  return;
}

/* NB: フォントの破棄は行われない事に注意する事！
       (フォントは他で使い回す可能性がある為) */
void ddddd_ta_free(TextArea *ta)
{
  SDL_FreeSurface(ta.surface);
  return;
}

void ddddd_ta_write(TextArea *ta, char[] str)
{
  // lfで分割する
  char[][] lines = splitlines(str);
  bool is_first = true;
  foreach (char[] line; lines) {
    if (is_first) {
      // 一回目は改行無し
      is_first = false;
    }
    else {
      // 二回目以降は改行を入れる
      ddddd_ta_lf(ta);
    }
    // 文字列が空なら何もしない
    if (!line.length) continue;
    // 不要な制御コードを除去する(ゴミを出力させない為)
    // TODO: あとで
    char[] cleanline = line;
    char *clz = toStringz(cleanline);
    // 出力領域のサイズを取得する
    int w, h;
    int r = TTF_SizeUTF8(ta.font, clz, &w, &h);
    if (r < 0) throw new Error("TTF_SizeUTF8 error");
    // 一行ずつレンダリングする
    SDL_Surface *sr;
    if (ta.flags & RENDERTYPE_SOLID) {
      sr = TTF_RenderUTF8_Solid(ta.font, clz, ta.fgcolor);
    }
    else if (ta.flags & RENDERTYPE_SHADED) {
      sr = TTF_RenderUTF8_Shaded(ta.font, clz, ta.fgcolor, ta.bgcolor);
    }
    else if (ta.flags & RENDERTYPE_BLENDED) {
      sr = TTF_RenderUTF8_Blended(ta.font, clz, ta.fgcolor);
    }
    else {
      throw new Error("invalid rendertype");
    }
    if (!sr) throw new Error("TTF_RenderUTF8_* error");
    // srを転送する
    SDL_Rect rect;
    rect.w = cast(Uint16)w;
    rect.h = cast(Uint16)h;
    rect.x = cast(Sint16)ta.locate_x;
    rect.y = cast(Sint16)ta.locate_y;
    SDL_BlitSurface(sr, null, ta.surface, &rect);
    // srを解放する
    SDL_FreeSurface(sr);
    // カーソルを移動させる
    ta.locate_x += w;
    ta.locate_y += h;
  }

  return;
}

void ddddd_ta_writeln(TextArea *ta, char[] str)
{
  return ddddd_ta_write(ta, str ~ "\n");
}


/* vim:set ft=d sw=2 ts=8 sts=2 et cindent: */
