/* $Id$ */

module denzi_image2;

/* このモジュールは、以下の各画像パーツの情報を保持する。
   - Denzi071006-1.png ( IMG_PILLARS )
   - Denzi071006-2.png ( IMG_WALLS )
   - Denzi071006-5.png ( IMG_DOORS )
   - Denzi071009-1.png ( IMG_MISC )
   - Denzi071005-2.png ( IMG_DEBUG )
 */

// TODO: 各関数を一つにまとめる(引数にIMG_*を受け取るようにする)

private import SDL;
private import SDL_image;
private import std.string;
private import ddddd.ddddd;


public enum {
  TILE_WIDTH = 32, // 床タイルのベース横幅
  TILE_HEIGHT = 16, // 床タイルのベース縦幅
};

public enum {
  IMG_PILLARS = 0, // 円柱ファイル(96x96)
  IMG_WALLS   = 1, // 壁ファイル(96x96)
  IMG_DOORS   = 2, // ドア壁、アーチ、ギミック壁のファイル(64x80, 64x96)
  IMG_MISC    = 3, // 床とその他のファイル(16x8)
  IMG_DEBUG   = 4, // デバッグ用壁ファイル(96x96)
};

private const static char[] IMG_PILLARS_FILENAME = "Denzi071006-1.png";
private const static char[] IMG_WALLS_FILENAME   = "Denzi071006-2.png";
private const static char[] IMG_DOORS_FILENAME   = "Denzi071006-5.png";
private const static char[] IMG_MISC_FILENAME    = "Denzi071009-1.png";
private const static char[] IMG_DEBUG_FILENAME   = "Denzi071005-2.png";

public struct DIInfo {
  int type; // IMG_PILLARS類
  SDL_Rect *rect; // IMG_PILLARS等の中の、該当画像のrect
  int qv_width; // 右上を上辺として見た際の横ブロック数。床は1、他は大体3。
  int qv_height; // 右上を上辺として見た際の縦ブロック数。床は1、他は大体3。
  int coordinate_x; // 前述rect内の中央ブロックの(画像での)左上座標のx
  int coordinate_y; // 前述rect内の中央ブロックの(画像での)左上座標のy
};


private SDL_Surface *_load_di_one(char[] filepath)
{
  SDL_Surface *tile = IMG_Load(toStringz(filepath));
  if (!tile) {
    throw new Error("failed load tile " ~ filepath);
  }
  SDL_LockSurface(tile);
  SDL_SetColorKey(tile, SDL_SRCCOLORKEY, ddddd_getPixel(tile, 0, 0));
  SDL_UnlockSurface(tile);
  return tile;
}

public SDL_Surface *[]load_di(char[] datadir = "./")
{
  SDL_Surface *[]di_tile;
  //di_tile.length = 5;
  di_tile.length = 4;
  di_tile[IMG_PILLARS] = _load_di_one(datadir ~ "/" ~ IMG_PILLARS_FILENAME);
  di_tile[IMG_WALLS] = _load_di_one(datadir ~ "/" ~ IMG_WALLS_FILENAME);
  di_tile[IMG_DOORS] = _load_di_one(datadir ~ "/" ~ IMG_DOORS_FILENAME);
  di_tile[IMG_MISC] = _load_di_one(datadir ~ "/" ~ IMG_MISC_FILENAME);
  //di_tile[IMG_DEBUG] = _load_di_one(datadir ~ "/" ~ IMG_DEBUG_FILENAME);
  return di_tile;
}

/* 床画像(32x16)のrectを返す */
public DIInfo *get_rect_of_floor(int index)
{
  if (index < 1) return null;
  if (table_of_floor.length < index) return null;
  return _table2diinfo(IMG_MISC, table_of_floor, index);
}
/* indexの番号と実際の画像の対応表
 *    1  2  3  4  5
 * 6  7  8  9 10 11
 *
 *                   12 13
 * (※12は厳密には床ではなく、壁等の上部(断面？)だと思われる)
 * (※13はカーソルとして利用する)
 */
private const static Sint16[][] table_of_floor = [
  // width, height, x, y, qv_width, qv_height, coordinate_x, coordinate_y
  null,
  [ 32, 16, 32*1, 0,    1, 1, 0, 0 ],
  [ 32, 16, 32*2, 0,    1, 1, 0, 0 ],
  [ 32, 16, 32*3, 0,    1, 1, 0, 0 ],
  [ 32, 16, 32*4, 0,    1, 1, 0, 0 ],
  [ 32, 16, 32*5, 0,    1, 1, 0, 0 ],

  [ 32, 16, 32*0, 16,   1, 1, 0, 0 ],
  [ 32, 16, 32*1, 16,   1, 1, 0, 0 ],
  [ 32, 16, 32*2, 16,   1, 1, 0, 0 ],
  [ 32, 16, 32*3, 16,   1, 1, 0, 0 ],
  [ 32, 16, 32*4, 16,   1, 1, 0, 0 ],
  [ 32, 16, 32*5, 16,   1, 1, 0, 0 ],

  [ 32, 16, 32*6, 16*4, 1, 1, 0, 0 ],
  [ 32, 16, 32*7, 16*4, 1, 1, 0, 0 ],
];

/* 壁画像(96x96)のrectを返す(テーブルは円柱と共用) */
public DIInfo *get_rect_of_wall(int index)
{
  if (index < 1) return null;
  if (table_of_wall.length < index) return null;
  return _table2diinfo(IMG_WALLS, table_of_wall, index);
}

/* 円柱画像(96x96)のrectを返す(テーブルは壁と共用) */
public DIInfo *get_rect_of_pillar(int index)
{
  if (index < 1) return null;
  if (table_of_wall.length < index) return null;
  return _table2diinfo(IMG_PILLARS, table_of_wall, index);
}
/* indexの番号と実際の画像の対応表
 *  1  2  3  4
 *  5  6  7  8
 *  9 10 11 12
 * 13 14 15 16
 */
private const static Sint16[][] table_of_wall = [
  // width, height, x, y, qv_width, qv_height, coordinate_x, coordinate_y
  null,
  [ 96, 96, 96*0, 96*0, 3, 3, (96/2)-(TILE_WIDTH/2), 48+(TILE_HEIGHT/2) ],
  [ 96, 96, 96*1, 96*0, 3, 3, (96/2)-(TILE_WIDTH/2), 48+(TILE_HEIGHT/2) ],
  [ 96, 96, 96*2, 96*0, 3, 3, (96/2)-(TILE_WIDTH/2), 48+(TILE_HEIGHT/2) ],
  [ 96, 96, 96*3, 96*0, 3, 3, (96/2)-(TILE_WIDTH/2), 48+(TILE_HEIGHT/2) ],
  [ 96, 96, 96*0, 96*1, 3, 3, (96/2)-(TILE_WIDTH/2), 48+(TILE_HEIGHT/2) ],
  [ 96, 96, 96*1, 96*1, 3, 3, (96/2)-(TILE_WIDTH/2), 48+(TILE_HEIGHT/2) ],
  [ 96, 96, 96*2, 96*1, 3, 3, (96/2)-(TILE_WIDTH/2), 48+(TILE_HEIGHT/2) ],
  [ 96, 96, 96*3, 96*1, 3, 3, (96/2)-(TILE_WIDTH/2), 48+(TILE_HEIGHT/2) ],
  [ 96, 96, 96*0, 96*2, 3, 3, (96/2)-(TILE_WIDTH/2), 48+(TILE_HEIGHT/2) ],
  [ 96, 96, 96*1, 96*2, 3, 3, (96/2)-(TILE_WIDTH/2), 48+(TILE_HEIGHT/2) ],
  [ 96, 96, 96*2, 96*2, 3, 3, (96/2)-(TILE_WIDTH/2), 48+(TILE_HEIGHT/2) ],
  [ 96, 96, 96*3, 96*2, 3, 3, (96/2)-(TILE_WIDTH/2), 48+(TILE_HEIGHT/2) ],
  [ 96, 96, 96*0, 96*3, 3, 3, (96/2)-(TILE_WIDTH/2), 48+(TILE_HEIGHT/2) ],
  [ 96, 96, 96*1, 96*3, 3, 3, (96/2)-(TILE_WIDTH/2), 48+(TILE_HEIGHT/2) ],
  [ 96, 96, 96*2, 96*3, 3, 3, (96/2)-(TILE_WIDTH/2), 48+(TILE_HEIGHT/2) ],
  [ 96, 96, 96*3, 96*3, 3, 3, (96/2)-(TILE_WIDTH/2), 48+(TILE_HEIGHT/2) ],
];



private DIInfo *_table2diinfo(int imgtype, Sint16[][] table, int index)
{
  if (index < 1) return null;
  if (table.length <= index) return null;
  Sint16[] data = table[index];
  if (!data) return null;
  DIInfo *diinfo = new DIInfo;
  SDL_Rect *rect = new SDL_Rect;
  rect.w = cast(Uint16)table[index][0];
  rect.h = cast(Uint16)table[index][1];
  rect.x = cast(Sint16)table[index][2];
  rect.y = cast(Sint16)table[index][3];
  diinfo.type = imgtype;
  diinfo.rect = rect;
  diinfo.qv_width = table[index][4];
  diinfo.qv_height = table[index][5];
  diinfo.coordinate_x = table[index][6];
  diinfo.coordinate_y = table[index][7];
  return diinfo;
}

/* マップを三倍に広くする。空いた隙間にはnullが入る。
   床タイルに対して、立体物タイルは全て三倍のサイズなので、
   予めこれにかけてから表示させると丁度良くなる。 */
public void *[][] triplicate_map(void *[][] src)
{
  if (!src.length) return [];
  void *[][] result;
  result.length = src.length*3;
  for (int y = 0; y < src.length; y++) {
    result[y*3+0].length = src[y].length*3;
    result[y*3+1].length = src[y].length*3;
    result[y*3+2].length = src[y].length*3;
    for (int x = 0; x < src[y].length; x++) {
      result[y*3+0][x*3+0] = null;
      result[y*3+0][x*3+1] = null;
      result[y*3+0][x*3+2] = null;
      result[y*3+1][x*3+0] = null;
      result[y*3+1][x*3+1] = src[y][x];
      result[y*3+1][x*3+2] = null;
      result[y*3+2][x*3+0] = null;
      result[y*3+2][x*3+1] = null;
      result[y*3+2][x*3+2] = null;
    }
  }
  return result;
}

/* ドア画像(64x80/64x112)のrectを返す */
public DIInfo *get_rect_of_door(int index)
{
  if (index < 1) return null;
  if (table_of_door.length < index) return null;
  return _table2diinfo(IMG_DOORS, table_of_door, index);
}
/* indexの番号と実際の画像の対応表
 *  1  2  3  4
 *  5  6
 *  7  8  9 10
 */
private const static Sint16[][] table_of_door = [
  // width, height, x, y, qv_width, qv_height, coordinate_x, coordinate_y
  null,
  [ 64, 80,  64*0, 80*0,   3, 3, 16, 8*7 ],
  [ 64, 80,  64*1, 80*0,   3, 3, 16, 8*7 ],
  [ 64, 80,  64*2, 80*0,   3, 3, 16, 8*7 ],
  [ 64, 80,  64*3, 80*0,   3, 3, 16, 8*7 ],
  [ 64, 80,  64*0, 80*1,   3, 3, 16, 8*7 ],
  [ 64, 80,  64*1, 80*1,   3, 3, 16, 8*7 ],
  [ 64, 104, 64*0, 80*2,   3, 3, 16, 104 - 8*3 ],
  [ 64, 104, 64*1, 80*2,   3, 3, 16, 104 - 8*3 ],
  [ 64, 80,  64*2, 80*2+8, 3, 3, 16, 8*7 ],
  [ 64, 80,  64*3, 80*2+8, 3, 3, 16, 8*7 ],
];



/* vim:set ft=d sw=2 ts=8 sts=2 et cindent: */
